local Lib = MissionMinder;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = Lib.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = Lib.debug;

Lib.hooks = {};
local module = Lib.hooks;

local function startMission(self)
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage
	local mission = MissionPage.missionInfo
	if (not mission) then
		return;
	end

	C_Garrison.StartMission(mission.missionID);
	PlaySound("UI_Garrison_CommandTable_MissionStart");
	Lib.callbacks:Fire("MISSION_STARTED", mission.missionID)

	GarrisonMissionList_UpdateMissions();
	GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
	GarrisonMissionPage_Close();
	if (not GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_GARRISON_LANDING)) then
		GarrisonLandingPageTutorialBox:Show();
	end
end

local function missionCompleted(self, missionID, canComplete, succeeded)
	Lib.callbacks:Fire("MISSION_COMPLETED", missionID, succeeded)
end

local function missionPageAddFollower(frame, info)
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage;
	local mission = MissionPage.missionInfo

	if ( mission and frame and frame.info) then
		Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, frame:GetID(), frame.info.followerID, "MissionPage")
	end
end

local function missionPageRemoveFollower(frame)
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage;
	local mission = MissionPage.missionInfo

	if ( mission and frame) then
		Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, frame:GetID(), "MissionPage")
	end
end

local function showMissionPage(mission)
	Lib.callbacks:Fire("MISSION_SELECTED", mission and mission.missionID or nil);
end

local function closeMissionPage()
	Lib.callbacks:Fire("MISSION_SELECTED", nil);
end



--[===[@alpha@
local function debugEvent(...)
	print("[EVENT]", ...)
end
--@end-alpha@]===]

local function initialize()
	DEBUG("   |cffffffffHooks|r initialized");

	-- MISSION_STARTED
	GarrisonMissionFrame.MissionTab.MissionPage.StartMissionButton:SetScript("OnClick", startMission);

	-- MISSION_COMPLETED
	local events = Lib.events
	events:RegisterEvent("hooks", "GARRISON_MISSION_COMPLETE_RESPONSE", missionCompleted)

	-- Set party follower frame IDs to track actions
	local partyFollowers = GarrisonMissionFrame.MissionTab.MissionPage.Followers;
	for i = 1, #partyFollowers do
		partyFollowers[i]:SetID(i)
	end

	-- FOLLOWER_ADDED_TO_MISSION
	hooksecurefunc("GarrisonMissionPage_SetFollower", missionPageAddFollower)

	-- FOLLOWER_REMOVED_FROM_MISSION
	hooksecurefunc("GarrisonMissionPage_ClearFollower", missionPageRemoveFollower)

	-- MISSION_SELECTED
	hooksecurefunc("GarrisonMissionPage_ShowMission", showMissionPage)
	hooksecurefunc("GarrisonMissionPage_Close", closeMissionPage)

	--[===[@alpha@
	Lib.RegisterCallback("_hooks", "MISSION_STARTED", debugEvent)
	Lib.RegisterCallback("_hooks", "MISSION_COMPLETED", debugEvent)
	Lib.RegisterCallback("_hooks", "FOLLOWER_ADDED_TO_MISSION", debugEvent)
	Lib.RegisterCallback("_hooks", "FOLLOWER_REMOVED_FROM_MISSION", debugEvent)
	Lib.RegisterCallback("_hooks", "MISSION_SELECTED", debugEvent)
	--@end-alpha@]===]
end

local events = Lib.events
events:RegisterEvent("hooks", "ADDON_LOADED", initialize);
