local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

print("button.lua");

--Map of arrays to each missions party layout
--Used to determine the position that each follower is displayed within a party
-- parties[misisonID][1..3]
local parties = {}

module.parties = parties

local function getFollowerMissionBiasRGB(missionID,followerID)
	local bias = C_Garrison.GetFollowerBiasForMission(missionID, followerID);

	if ( bias == -1 ) then -- Impossible
		return 1, 0.1, 0.1;
	elseif ( bias < 0 ) then -- Hard
		return 1, 0.5, 0.25;
	else -- Normal
		return 1, 1, 1;
	end
end

local function GarrisonMissionPlanner_CheckCounter(button, counterID)
	for i = 1, #button.Mechanics do
		if ( counterID == button.Mechanics[i].mechanicID and not button.Mechanics[i].hasCounter ) then
			button.Mechanics[i].hasCounter = true;
			button.Mechanics[i].Check:Show();
			return;
		end
	end
end

function GarrisonMissionPlannerButton_OnClick(self, button)
	local mission = self.info;
	if ( IsModifiedClick("CHATLINK") ) then
		local missionLink = C_Garrison.GetMissionLink(mission.missionID);
		if (missionLink) then
			ChatEdit_InsertLink(missionLink);
		end
		return;
	end

	PlaySound("UI_Garrison_CommandTable_SelectMission");
	GarrisonMissionPlanner_SelectMission(mission)
end

function GarrisonMissionPlannerButton_OnDblClick(self, button)
	if ( button == "LeftButton" ) then
		-- Suggest followers
	elseif ( button == "RightButton" ) then
		clearFollowers(self.info);
	end
end

function GarrisonMissionPlannerMechanic_OnEnter(self)
	if ( self.info ) then
		GarrisonMissionMechanic_OnEnter(self)
	elseif ( self.environment ) then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(self.environment);
		GameTooltip:AddLine(self.environmentDesc, 1, 1, 1, 1);
		GameTooltip:Show();
	end
end

function GarrisonMissionPlannerMechanic_OnLeave(self)
	GarrisonMissionMechanicTooltip:Hide();
	GameTooltip:Hide()
end

function GarrisonMissionPlannerFollowerFrame_OnMouseUp(self, button)
	GarrisonMissionPlanner_SelectMission(self:GetParent().info)
	if ( button == "RightButton" ) then
		GarrisonMissionPlanner_ClearFollower(self, true);
	end
end

function module.clearFollowers(mission)
	for i = 1, #mission.followers do
		local followerID = mission.followers[i];
		C_Garrison.RemoveFollowerFromMission(mission.missionID, followerID)
		Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, i, "MultiPlanner")
	end
	parties[mission.missionID] = {}
end

function GarrisonMissionPlanner_SetRewards(self, rewards, numRewards)
	if (numRewards > 0) then
		local index = 1;
		for id, reward in pairs(rewards) do
			if (not self.Rewards[index]) then
				self.Rewards[index] = CreateFrame("Frame", nil, self, "MultiPlannerListButtonRewardTemplate");
				self.Rewards[index]:SetPoint("RIGHT", self.Rewards[index-1], "LEFT", -4, 0);
			end
			local Reward = self.Rewards[index];
			Reward.Quantity:Hide();
			Reward.IconBorder:Hide();
			Reward.itemID = nil;
			Reward.currencyID = nil;
			Reward.tooltip = nil;
			if (reward.itemID) then
				Reward.itemID = reward.itemID;
				GarrisonMissionFrame_SetItemRewardDetails(Reward);

				local _, _, quality = GetItemInfo(reward.itemID);
				if  ( quality and quality > LE_ITEM_QUALITY_COMMON and BAG_ITEM_QUALITY_COLORS[quality]) then
					Reward.IconBorder:Show();
					Reward.IconBorder:SetVertexColor(BAG_ITEM_QUALITY_COLORS[quality].r, BAG_ITEM_QUALITY_COLORS[quality].g, BAG_ITEM_QUALITY_COLORS[quality].b);
				end

				if ( reward.quantity > 1 ) then
					Reward.Quantity:SetText(reward.quantity);
					Reward.Quantity:Show();
				end
			else
				Reward.Icon:SetTexture(reward.icon);
				Reward.title = reward.title
				if (reward.currencyID and reward.quantity) then
					if (reward.currencyID == 0) then
						Reward.tooltip = GetMoneyString(reward.quantity);
						Reward.Quantity:SetText(BreakUpLargeNumbers(math.floor(reward.quantity / COPPER_PER_GOLD)));
						Reward.Quantity:Show();
					else
						Reward.currencyID = reward.currencyID;
						Reward.Quantity:SetText(reward.quantity);
						Reward.Quantity:Show();
					end
				else
					Reward.tooltip = reward.tooltip;
				end

				if ( reward.followerXP and reward.followerXP > 0 ) then
					Reward.Quantity:SetText(BreakUpLargeNumbers(reward.followerXP));
					Reward.Quantity:Show();
				end
			end
			Reward:Show();
			index = index + 1;
		end
	end

	for i = (numRewards + 1), #self.Rewards do
		self.Rewards[i]:Hide();
	end
end

function GarrisonMissionPlanner_SetFollowerPortrait(portraitFrame, followerInfo, showItemLevel)
	local color = ITEM_QUALITY_COLORS[followerInfo.quality];
	portraitFrame.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
	portraitFrame.LevelBorder:SetVertexColor(color.r, color.g, color.b);
	if ( showItemLevel and followerInfo.level == GarrisonMissionFrame.followerMaxLevel ) then
		portraitFrame.Level:SetFormattedText(GARRISON_FOLLOWER_ITEM_LEVEL, followerInfo.iLevel);
	else
		portraitFrame.Level:SetText(followerInfo.level);
	end
	if ( followerInfo.displayID ) then
		GarrisonFollowerPortrait_Set(portraitFrame.Portrait, followerInfo.portraitIconID);
	end
end


local function updateMissionForParty(button)
	local mission = button.info
	if ( not mission ) then
		return
	end

	local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier = C_Garrison.GetPartyMissionInfo(mission.missionID);

	-- TIME
	if ( isMissionTimeImproved ) then
		totalTimeString = GREEN_FONT_COLOR_CODE..totalTimeString..FONT_COLOR_CODE_CLOSE;
	elseif ( totalTimeSeconds >= GARRISON_LONG_MISSION_TIME ) then
		totalTimeString = format(GARRISON_LONG_MISSION_TIME_FORMAT, totalTimeString);
	end
	button.Summary:SetFormattedText(PARENS_TEMPLATE, totalTimeString);

	-- SUCCESS CHANCE
	button.Chance:SetFormattedText(PERCENTAGE_STRING, successChance);

	-- ENVIRONMENT
	if ( isEnvMechanicCountered ) then
		button.MissionEnvIcon.Check:Show();
	else
		button.MissionEnvIcon.Check:Hide();
	end

	-- MECHANICS
	for i = 1, #button.Mechanics do
		button.Mechanics[i].hasCounter = nil;
		button.Mechanics[i].Check:Hide();
	end

	for i = 1, #button.Followers do
		local frame = button.Followers[i]
		local follower = frame.info
		if ( follower ) then
			if (C_Garrison.GetFollowerBiasForMission(mission.missionID, follower.followerID) > -1 ) then
				if (not follower.abilities) then
					follower.abilities = C_Garrison.GetFollowerAbilities(follower.followerID)
				end
				for a = 1, #follower.abilities do
					local ability = follower.abilities[a];
					for counterID, counterInfo in pairs(ability.counters) do
						GarrisonMissionPlanner_CheckCounter(button, counterID);
					end
				end
			end
		end
	end

	if 1 then -- TODO: Need to finish this implementation
		return;
	end

	-- XP
	if ( xpBonus > 0 ) then
		rewardsFrame.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP_PLUS, MISSION_PAGE_FRAME.xp + xpBonus, xpBonus);
	else
		rewardsFrame.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP, MISSION_PAGE_FRAME.xp);
	end

	-- Material
	GarrisonMissionPage_UpdateRewardQuantities(rewardsFrame, materialMultiplier);
end

local counterCache = {};
local function getFollowerCounters(missionID, followerID)
	if ( not missionID or not followerID ) then
		return nil
	end

	if ( not counterCache[missionID]) then
		counterCache[missionID] = C_Garrison.GetBuffedFollowersForMission(missionID);

		for id, traits in pairs(C_Garrison.GetFollowersTraitsForMission(missionID)) do
			if ( not counterCache[missionID][id] ) then
				counterCache[missionID][id] = traits
			else
				for i=1, #traits do
					tinsert(counterCache[missionID][id], traits[i])
				end
			end
		end
	end

	return counterCache[missionID][followerID];
end

local space=6 -- TODO: remove temporary variable and finalize calculations
local counterLayout = {
	[1] = {
		[1] = { "CENTER", 22, 0 }
	},

	[2] = {
		[1] = { "CENTER", 22-10-(space/2), 0 },
		[2] = { "TOPLEFT", 1, "TOPRIGHT", space, 0 },
	},

	[3] = {
		[1] = { "CENTER", 22-10-(space/2), 10+(space/2) },
		[2] = { "TOPLEFT", 1, "TOPRIGHT", space, 0 },
		[3] = { "TOPLEFT", 1, "BOTTOMLEFT", 0, -space },
	},

	[4] = {
		[1] = { "CENTER", 22-10-(space/2), 10+(space/2) },
		[2] = { "TOPLEFT", 1, "TOPRIGHT", space, 0 },
		[3] = { "TOPLEFT", 1, "BOTTOMLEFT", 0, -space },
		[4] = { "TOPLEFT", 3, "TOPRIGHT", space, 0 }
	}
}

local function applyCounterLayout(counter, layout, i)
	layout = layout[i]
	counter[i]:ClearAllPoints()

	if ( #layout == 3) then
		counter[i]:SetPoint(layout[1], layout[2], layout[3]);
	elseif ( #layout == 4) then
		counter[i]:SetPoint(layout[1], frame:GetParent(), layout[2], layout[3], layout[4]);
	elseif ( #layout == 5) then
		counter[i]:SetPoint(layout[1], counter[layout[2]], layout[3], layout[4], layout[5]);
	end
end

local function updateFollowerCounters(frame, counters)
	if (counters) then
		local layout = counterLayout[#counters]
		for i = 1, #counters do
			if (not frame.Counters[i]) then
				frame.Counters[i] = CreateFrame("Frame", nil, frame, "GarrisonMissionAbilityCounterTemplate");
			end
			local Counter = frame.Counters[i];
			Counter.info = counters[i];
			Counter.info.showCounters = true;
			Counter.Icon:SetTexture(counters[i].icon);
			Counter.tooltip = counters[i].name;
			applyCounterLayout(frame.Counters, layout, i)
			Counter:Show();

			if( counters[i].traitID ) then
				Counter.Border:Hide()
			else
				Counter.Border:Show()
			end
		end
		for i = (#counters + 1), #frame.Counters do
			frame.Counters[i]:Hide();
		end
	else
		for i=1, #frame.Counters do
			frame.Counters[i]:Hide();
		end
	end
end

local function updateFollowerIcon(mission, frame, followerID)
	if ( followerID == nil ) then
		frame.info = nil;
		frame.PortraitFrame.Empty:Show();
		frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
		frame.PortraitFrame.LevelBorder:SetWidth(58);
		frame.PortraitFrame.Level:SetText("");
	else
		local follower = C_Garrison.GetFollowerInfo(followerID);
		frame.info = follower;
		local showItemLevel;
		if ( mission.iLevel > 600 and follower.level == GarrisonMissionFrame.followerMaxLevel ) then
			frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_iLvlBorder");
			frame.PortraitFrame.LevelBorder:SetWidth(70);
			showItemLevel = true;
		else
			frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
			frame.PortraitFrame.LevelBorder:SetWidth(58);
			showItemLevel = false;
		end

		GarrisonMissionPlanner_SetFollowerPortrait(frame.PortraitFrame, follower, showItemLevel);
		frame.PortraitFrame.Empty:Hide();

		local r,g,b = getFollowerMissionBiasRGB(mission.missionID, follower.followerID);
		frame.PortraitFrame.Level:SetTextColor(r, g, b);
	end

	updateFollowerCounters(frame, getFollowerCounters(mission.missionID, followerID));
end

local function updateMissionMechanics(button)
	local mission = button.info

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	button.MissionEnvIcon.Texture:SetTexture(environmentTexture);
	button.MissionEnvIcon.environment = environment
	button.MissionEnvIcon.environmentDesc = environmentDesc

	local numMechs = 0
	for i = 1, #enemies do
		local enemy = enemies[i]
		for id, mechanic in pairs(enemy.mechanics) do
			numMechs = numMechs + 1;
			if (not button.Mechanics[numMechs]) then
				button.Mechanics[numMechs] = CreateFrame("Frame", nil, button, "PlannerMechanicTemplate");
				button.Mechanics[numMechs]:SetPoint("RIGHT", button.Mechanics[numMechs-1], "LEFT", -2, 0);
			end
			local Mechanic = button.Mechanics[numMechs];
			Mechanic.info = mechanic;
			Mechanic.mechanicID = id;
			Mechanic.Texture:SetTexture(mechanic.icon);
			Mechanic:Show();
		end
	end
	for j=(numMechs + 1), #button.Mechanics do
		button.Mechanics[j]:Hide();
		button.Mechanics[j].mechanicID = nil;
		button.Mechanics[j].info = nil;
	end
end

local function updateMissionButton(button)
	local mission = button.info
	if ( mission ) then
		button.Title:SetWidth(0);
		button.Title:SetText(mission.name);

		if ( mission.durationSeconds >= GARRISON_LONG_MISSION_TIME ) then
			local duration = format(GARRISON_LONG_MISSION_TIME_FORMAT, mission.duration);
			button.Summary:SetFormattedText(PARENS_TEMPLATE, duration);
		else
			button.Summary:SetFormattedText(PARENS_TEMPLATE, mission.duration);
		end
		if ( button.Title:GetWidth() + button.Summary:GetWidth() + 8 < 655 - mission.numRewards * 65 ) then
			button.Title:SetPoint("LEFT", 165, 0);
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("BOTTOMLEFT", button.Title, "BOTTOMRIGHT", 8, 0);
		else
			button.Title:SetPoint("LEFT", 165, 10);
			button.Title:SetWidth(655 - mission.numRewards * 65);
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);
		end
		if ( mission.locPrefix ) then
			button.LocBG:Show();
			button.LocBG:SetAtlas(mission.locPrefix.."-List");
		else
			button.LocBG:Hide();
		end
		if (mission.isRare) then
			button.RareOverlay:Show();
			button.RareText:Show();
			button.IconBG:SetVertexColor(0, 0.012, 0.291, 0.4)
		else
			button.RareOverlay:Hide();
			button.RareText:Hide();
			button.IconBG:SetVertexColor(0, 0, 0, 0.4)
		end

		button.Icon:SetAtlas(mission.typeAtlas)

		local showingItemLevel = false;
		if ( mission.level == GARRISON_FOLLOWER_MAX_LEVEL and mission.iLevel > 0 ) then
			showingItemLevel = true;
		end
		button.Level:SetText(showingItemLevel and mission.iLevel or mission.level);

		button.Chance:SetPoint("CENTER", button, "LEFT", 40, (mission.isRare) and 0 or -10);

		for i = 1, mission.numFollowers do
			button.Followers[i]:Show();
			if ( not parties[mission.missionID] ) then
				parties[mission.missionID] = {}
			end
			updateFollowerIcon(mission, button.Followers[i], parties[mission.missionID][i]);
		end
		for i = mission.numFollowers + 1, #button.Followers do
			button.Followers[i]:Hide()
		end

		local missionPage = GarrisonMissionFrame.MissionTab.MissionPage;
		if ( missionPage.missionInfo and missionPage.missionInfo.missionID == mission.missionID ) then
			button.SelectedBox:Show();
			button:Disable()
		else
			button.SelectedBox:Hide();
			button:Enable()
		end

		GarrisonMissionPlanner_SetRewards(button, mission.rewards, mission.numRewards);

		--updateFollowersForMission(button)
		updateMissionMechanics(button)
		updateMissionForParty(button)

		button:Show();
	else
		button:Hide();
	end
end

function module.markForUpdate(button)
	button.dirty = true
end

function GarrisonMisisonPlannerButton_OnUpdate(self)
	if ( self.dirty ) then
		updateMissionButton(self)
		self.dirty = nil
	end
end

local function addFollowerToMission(event, missionID, index, followerID, source)
	if ( not parties[missionID] ) then
		parties[missionID] = {}
	end
	parties[missionID][index] = followerID;
end

local function removeFollowerFromMission(event, missionID, index, source)
	if ( parties[missionID] ) then
		parties[missionID][index] = nil;
	end
end

-- Allow changes on the MissionPage to be reflected in the multiplanner
Lib.RegisterCallback("multiplanner", "FOLLOWER_ADDED_TO_MISSION", addFollowerToMission)
Lib.RegisterCallback("multiplanner", "FOLLOWER_REMOVED_FROM_MISSION", removeFollowerFromMission)
