-- Drag and drop controls for multiplanner
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local OPTIONS = MissionMinder.options;
local DEBUG = MissionMinder.debug;
local addon = MissionMinder.multiplanner
print("dragndrop.lua");

local sourceFrame = nil;

local function followerStopDrag(self)
	if (GarrisonFollowerPlacer:IsShown()) then
		GarrisonFollowerPlacerFrame:Show();
	end
end

local function followerStartDrag(self, button)
	sourceFrame = nil;

	if ( not GarrisonMissionFrame.PlannerTab:IsShown() ) then
		return;
	end

	if ( self.info.status or not self.info.isCollected ) then
		return;
	end

	CloseDropDownMenus();
	GarrisonMissionPlanner_SetFollowerPortrait(GarrisonFollowerPlacer, self.info);
	GarrisonFollowerPlacer.info = self.info;
	local cursorX, cursorY = GetCursorPosition();
	local uiScale = UIParent:GetScale();
	GarrisonFollowerPlacer:SetPoint("TOP", UIParent, "BOTTOMLEFT", cursorX / uiScale, cursorY / uiScale + 24);
	GarrisonFollowerPlacer:Show();
	GarrisonFollowerPlacer:SetScript("OnUpdate", GarrisonFollowerPlacer_OnUpdate);
end

local function placerFrame_OnClick(self, button)
DEBUG("placerFrame_OnClick");
	local missionPage = GarrisonMissionFrame.MissionTab.MissionPage;
	local buttons = GarrisonMissionFrame.PlannerTab.listScroll.buttons;

	if ( button == "LeftButton" ) then
		local followers = nil;
		local method = nil;
		if ( missionPage:IsShown() ) then
			followers = missionPage.Followers
			method = GarrisonMissionPageFollowerFrame_OnReceiveDrag
		else
			for i = 1, #buttons do
				if ( buttons[i]:IsMouseOver() ) then
					followers = buttons[i].Followers;
					method = GarrisonMissionPlannerFollowerFrame_OnReceiveDrag
					break;
				end
			end
		end

		if ( followers and method ) then
			for i = 1, #followers do
				local followerFrame = followers[i];
				if ( followerFrame:IsMouseOver() ) then
					method(followerFrame);
				end
			end
		end
	end

DEBUG("ClearMouse()");
	GarrisonMissionFrame_ClearMouse();
end

function GarrisonMissionPlannerFollowerFrame_OnDragStart(self)
	if ( not self.info ) then
		return;
	end

	GarrisonMissionPlanner_SetFollowerPortrait(GarrisonFollowerPlacer, self.info);
	GarrisonFollowerPlacer.info = self.info;
	local cursorX, cursorY = GetCursorPosition();
	local uiScale = UIParent:GetScale();
	GarrisonFollowerPlacer:SetPoint("TOP", UIParent, "BOTTOMLEFT", cursorX / uiScale, cursorY / uiScale + 24);
	GarrisonFollowerPlacer:Show();
	GarrisonFollowerPlacer:SetScript("OnUpdate", GarrisonFollowerPlacer_OnUpdate);
	GarrisonMissionPlanner_ClearFollower(self, true);

	sourceFrame = self
end

function GarrisonMissionPlannerFollowerFrame_OnDragStop(self)
	if (GarrisonFollowerPlacer:IsShown()) then
		GarrisonFollowerPlacerFrame:Show();
	end
end

function GarrisonMissionPlannerFollowerFrame_OnReceiveDrag(self)
	if ( GarrisonFollowerPlacer:IsVisible() and GarrisonFollowerPlacer.info ) then
		local follower = self.info
		GarrisonMissionPlanner_SetFollower(self, GarrisonFollowerPlacer.info);
		GarrisonMissionFrame_ClearMouse();

		-- A follower was already placed here, and the follower that is being placed came from another frame.
		--Lets switch them!
		if ( follower and sourceFrame ) then
			GarrisonMissionPlanner_SetFollower(sourceFrame, follower);
		end
	end
	sourceFrame = nil;

	GarrisonMissionPlanner_SelectMission(self:GetParent().info)
end


function addon.dragndrop_init()
	local followerButtons = GarrisonMissionFrame.FollowerList.listScroll.buttons;
	for i = 1, #followerButtons do
		followerButtons[i]:HookScript("OnDragStart", followerStartDrag);
		followerButtons[i]:HookScript("OnDragStop", followerStopDrag);
	end

	GarrisonFollowerPlacerFrame:SetScript("OnClick", placerFrame_OnClick);
end